/*
    file: mGDFAMinimize_TBF.cpp
    Project: miniGrail
    License: LGPLv2.1 http://www.gnu.org/licenses/lgpl-2.1.html

    Created by: by Rui Zhou
    Maintainer: Cezar Campeanu <ccampeanu@upei.ca>

	Modified by Brandon Stewart July 2015
	  - Added namespace minigrail


 This Algorithm will take a DFA and do the minimization
 using the very basic table filling algorithm

 Based on polymophism, this algorithm could also be valid
 for DFCAs if some methods are properly overloaded
*/

namespace minigrail
{

void mGDFAMinimize_TBF(mGDFA& dfa) //question  why can not const?
{
	//DEBUG ?
	bool debug = false;
	if (DEBUG_TRIGGERED(DEBUG_mGDFAMinimize_TBF) || DEBUG_TRIGGERED(DEBUG_ALL))
	{
		debug = true;
		std::cout << "Debug output for mGNFAtoDFA_Set(NFA, DFA) is enabled."
				<< std::endl;
	}

	//rename : to conanical number states
	//crucial to do many algorithms
	dfa.rename();

	//first attach
	dfa.attach();

	//gather the labels in the NFA
	mGSet<int> alphabets;
	dfa.gatherLabels(alphabets);
	if (debug)
	{
		std::cout << "Gathered alphabets" << alphabets << std::endl;
	}

	//get all the states
	mGSet<mGState> states;
	dfa.gatherStates(states);

	//get starts and finals
	mGSet<mGState> startStates = dfa.getStartStates();
	mGSet<mGState> finalStates = dfa.getFinalStates();

	//get the value of the finals
	mGSet<int> finalValues;

	for (int a = 0; a < finalStates.size(); a++)
		finalValues += finalStates[a].getNumber();

	//the table to work on
	mGArray<mGArray<int> > table; //1 means distinguishable, -1 undefined

	//initilize
	for (int a = 0; a < states.size() - 1; a++)
	{
		mGArray<int> row;
		table += row;
		for (int b = 0; b < a + 1; b++)
		{
			if ((finalValues.member(a + 1) >= 0 && finalValues.member(b) < 0)
					|| (finalValues.member(a + 1) < 0
							&& finalValues.member(b) >= 0)) // distiguisahble
			{
				table[a] += 1;
			}
			else
			{
				table[a] += -1;
			}

		}

	}
	if (debug)
	{
		std::cout << "Initial table" << table << std::endl;
	}

	//working on the loop
	bool done = false;

	while (!done)
	{
		done = true;
		//go through each grid
		for (int a = 0; a < states.size() - 1; a++)
		{
			for (int b = 0; b < a + 1; b++)
			{
				if (table[a][b] == 1)
				{
					continue;
				}

				//mGState start1(a+1);
				//mGState start2(b);
				mGState end1;
				mGState end2;

				for (int c = 0; c < alphabets.size(); c++)
				{
					dfa.getDestination(a + 1, alphabets[c], end1);
					dfa.getDestination(b, alphabets[c], end2);

					//distinguishibale states
					if ((end1.getNumber() == NULL_INT
							&& end2.getNumber() != NULL_INT)
							|| (end1.getNumber() != NULL_INT
									&& end2.getNumber() == NULL_INT))
					{
						done = false;
						table[a][b] = 1;
						break;
					}
					else if (!(end1 == end2))
					{
						//if end1 and end2 are distiguishable in table
						int s = (end1 > end2) ?
								end2.getNumber() : end1.getNumber();
						int m = (end1 > end2) ?
								end1.getNumber() : end2.getNumber();
						if (table[m - 1][s] == 1)
						{
							done = false;
							table[a][b] = 1;
							break;
						}

					}

				}

			}
		}

	}
	if (debug)
	{
		std::cout << "afterwards table" << table << std::endl;
	}

	//now, establish the sets for equivalent states

	mGSet<mGSet<mGState> > eq_sets;

	for (int a = 0; a < states.size() - 1; a++)
	{
		for (int b = 0; b < a + 1; b++)
		{
			if (table[a][b] == -1)
			{
				bool needNewSet = true;
				for (int c = 0; c < eq_sets.size(); c++)
				{
					if (eq_sets[c].member(a + 1) >= 0
							|| eq_sets[c].member(b) >= 0)
					{
						eq_sets[c] += (a + 1);
						eq_sets[c] += b;
						needNewSet = false;
					}

				}
				if (needNewSet)
				{
					mGSet<mGState> newSet;
					newSet += (a + 1);
					newSet += b;
					eq_sets += newSet;
				}

			}

		}
	}

	///make sure the sets are in order
	for (int a = 0; a < eq_sets.size(); a++)
	{
		eq_sets[a].heapSort();
	}

	//make a new DFA
	mGDFA newDFA;
	//add the information from the old to new one
	//replace every equivalent states

	//start
	for (int a = 0; a < dfa.getStartStates().size(); a++)
	{
		mGState temp = dfa.getStartStates()[a];
		for (int a = 0; a < eq_sets.size(); a++)
		{
			if (eq_sets[a].member(temp) > 0)
			{
				temp = eq_sets[a][0];
			}
		}
		newDFA.addStart(temp);
	}
	//final
	for (int a = 0; a < dfa.getFinalStates().size(); a++)
	{
		mGState temp = dfa.getFinalStates()[a];
		for (int a = 0; a < eq_sets.size(); a++)
		{
			if (eq_sets[a].member(temp) > 0)
			{
				temp = eq_sets[a][0];
			}
		}
		newDFA.addFinal(temp);
	}
	// transitions
	for (int a = 0; a < dfa.getTransitionSize(); a++)
	{
		mGTransition temp = dfa[a];

		for (int a = 0; a < eq_sets.size(); a++)
		{
			if (eq_sets[a].member(temp.getStart()) > 0)
			{
				temp.setStart(eq_sets[a][0]);
			}

			if (eq_sets[a].member(temp.getEnd()) > 0)
			{
				temp.setEnd(eq_sets[a][0]);
			}

		}

		newDFA.addTransition(temp);

	}

	dfa = newDFA;

}

}// namespace minigrail
