/*
    file: mGArray.cpp
    Project: miniGrail
    License: LGPLv2.1 http://www.gnu.org/licenses/lgpl-2.1.html

    Created by: by Rui Zhou
    Maintainer: Cezar Campeanu <ccampeanu@upei.ca>


	Modified by Brandon Stewart July 2015
	  - Added namespace minigrail
	  - Moved headers to top of file
 This is the implementation of methods declared in mGArray.h

*/

#include <iostream>
#include <algorithm>
#include <stdlib.h>

namespace minigrail
{


//constructor
template<class T>
mGArray<T>::mGArray()
{
	sorted = true;
}

//assign
template<class T>
mGArray<T>& mGArray<T>::operator=(const mGArray<T>& a)
{
	this->sorted = a.sorted;
	this->v = a.v;
	return *this;
}


//if mGArray Contains a element
//make sure the elements are comparable because of the binary search
template<class T>
int mGArray<T>::member(const T& elementToSearch) const
{
	bool debug = false;
	if (DEBUG_TRIGGERED(DEBUG_mGArray_member) || DEBUG_TRIGGERED(DEBUG_ALL))
	{
		debug = true;
		std::cout << "Debug output for mGArray<T>:: member is enabled."
				<< std::endl;
	}

	if (debug)
	{
		std::cout << "loking for" << elementToSearch << "in" << std::endl
				<< *this << std::endl;

	}

	if (v.size() == 0)
	{
		return -1;
	}

	if (sorted) //binary search
	{

		if (debug)
		{
			std::cout << "binary search" << std::endl;

		}
		//TODO: verify Grail+
		/*		int left = 0;
		 int right = v.size();
		 int index;


		 while (true)
		 {
		 index = (left + right) / 2;

		 if (v[index] == elementToSearch)
		 {
		 if (debug)
		 {
		 std::cout << "binary search returns" << (index)
		 << std::endl;
		 }

		 return index;
		 }

		 if (left == index) // for example, key is the last element
		 {
		 if ((index + 1) < v.size() && v[index + 1] == elementToSearch)
		 {
		 if (debug)
		 {
		 std::cout << "binary search returns" << (index + 1)
		 << std::endl;
		 }
		 return (index + 1);
		 }
		 else
		 {

		 if (debug)
		 {
		 std::cout << "binary search NOT FOUND" << (index)
		 << std::endl;
		 }

		 return -1;
		 } //not found!
		 }

		 if (v[index] < elementToSearch)
		 {
		 right = index;
		 }
		 else
		 {
		 left = index;
		 }

		 }*/

		typename vector<T>::const_iterator it; //xxx remember this lesson
		//for const vectro,, use cosnt iterator

		//VIT it;

		it = lower_bound(v.begin(), v.end(), elementToSearch);

		//Check that we have actually found the value.
		//If the requested value is missing
		//then we will have the value before where the requested value
		//would be inserted.
		//std::cout << "@" << *it;

		if (it == v.end() || !(*it == elementToSearch))
		{

			//element not found
			return -1;
		}
		else
		{
			//element found
			return int(it - v.begin());
		}

	}
	else //linear search
	{
		if (debug)
		{
			std::cout << "linear search" << std::endl;

		}
		for (int a = 0; a < v.size(); a++)
		{
			if (v[a] == elementToSearch)
			{
				if (debug)
				{
					std::cout << "linear search returns" << a << std::endl;
				}
				return a;
			}

		}

		return -1;

	}
}

//pluseq
template<class T>
mGArray<T>& mGArray<T>::operator+=(const T& e)
{
	v.push_back(e);

	sorted = false; //question: two statements, inline?

	//TODO: IF THE ARRAY IS SORTED AND WE ADD A ELEMET BIGGER THAN LAST ONE IN ARRAY
	//WE KEEP SORTED AS TRUE?

	return *this;
}

template<class T>
mGArray<T>& mGArray<T>::operator+=(const mGArray<T>& array)
{

	for (int a = 0; a < array.size(); a++)
	{
		v.push_back(array[a]);
	}
	sorted = false;
	return *this;
}

//minuseq
template<class T>
mGArray<T>& mGArray<T>::operator-=(const T& e)
{
	int index = member(e);
	if (index == -1)
		return *this;
	//only remove the first one
	v.erase(v.begin() + index);
	return *this;
}

//remove all
template<class T>
mGArray<T>& mGArray<T>::removeAll(const T& e)
{
	vector<T> temp;
	for (int a = 0; a < v.size(); a++)
	{
		if (v[a] != e)
			temp.push_back(v[a]);
	}
	v = temp;

	return *this;
}

//help method for quick sort
template<class T>
int mGArray<T>::compare(const void * a, const void * b)
{
	if (*(T*) a > *(T*) b)
		return 1;
	else if (*(T*) a < *(T*) b)
		return -1;
	else
		return 0;
}

//quick sort with STL sort

template<class T>
void mGArray<T>::quickSortWithSort()
{
	if (sorted)
		return;

	sort(v.begin(), v.end());
	sorted = true;
//question: operate on poniters? temp objects?

	/*
	 sort is quicker for simple data
	 ref:	http://en.wikipedia.org/wiki/Sort_(C%2B%2B)
	 */

}

//quick sort with stdlib qsort

template<class T>
void mGArray<T>::quickSortWithQsort()
{
	if (sorted)
		return;

	qsort(&(v[0]), v.size(), sizeof(T), compare);
	//question: efficiency ?

	//return *this;
}

//heap sort
template<class T>
void mGArray<T>::heapSort()
{
	if (sorted)
		return;

	make_heap(v.begin(), v.end());
	sort_heap(v.begin(), v.end());
	sorted = true;
	return;
}

//intersection
template<class T>
void mGArray<T>::getIntersection(const mGArray<T>& a1, const mGArray<T>& a2,
		mGArray<T>& inter)
{
	bool debug = false;
	if (DEBUG_TRIGGERED(DEBUG_mGArray_getIntersection)
			|| DEBUG_TRIGGERED(DEBUG_ALL))
	{
		debug = true;
		std::cout << "Debug output for mGArray<T>:: getIntersection is enabled."
				<< std::endl;
	}

	for (int a = 0; a < a1.size(); a++)
	{
		if (a2.member(a1[a]) >= 0)
			inter += a1[a];

	}

	if (debug)
	{

		std::cout << a1 << "intersct" << a2 << "=" << inter << std::endl;

	}

}

template<class T>
mGArray<T>& mGArray<T>::mergeSortedArrays(mGArray<T>& second)
{
	assert(this->sorted && second.sorted);

	vector<T> temp;

	merge(v.begin(), v.end(), second.v.begin(), second.v.end(),
			back_inserter(temp)); //  one more thing to remeber .... :P

	v = temp;

	return *this;
}

//unique
template<class T>
void mGArray<T>::unique_array()
{
	//refer
	//http://stackoverflow.com/questions/1041620/most-efficient-way-to-erase-duplicates-and-sort-a-c-vector
	//

	if (!sorted)
	{
		this->quickSortWithSort();
		//this->quickSortWithQsort();
		//this->heapSort();

	}

	v.erase(unique(v.begin(), v.end()), v.end());

}


} // namespace minigrail
