/*
    file: mGArray.h
    Project: miniGrail
    License: LGPLv2.1 http://www.gnu.org/licenses/lgpl-2.1.html

    Created by: by Rui Zhou
    Maintainer: Cezar Campeanu <ccampeanu@upei.ca>


	Modified by Brandon Stewart July 2015
	  - Added namespace minigrail


 This is the base class of miniGrail

 It is a dynamic array based on the vector class in the standard C++ library

 */

#ifndef _MGARRAY

#include <vector>



namespace minigrail
{

using namespace std;

template<class T>

class mGArray
{

protected:
	vector<T> v;
	bool sorted;

public:
	//typedef typename vector<T>::iterator VIT;

//constructor copy  and de constructor
	mGArray();
	mGArray(const mGArray<T>& a) :
			v(a.v), sorted(a.sorted)
	{
	}
//~mGArray(){/*nothing, yep, nothing*/}

//assign
	mGArray<T>& operator=(const mGArray&);

//size
	inline int size() const
	{
		return v.size();
	}

//clear
	inline void clear()
	{
		v.clear();
		sorted=true;
	}

//whether sorted
	inline bool isSorted() const
	{
		return sorted;
	}

	inline void setSorted(bool s)
	{
		this->sorted = s;
	}

//get element
	inline T& operator[](int i)
	{
		return v[i];
	}

//get element with const
	inline const T& operator[](int i) const
	{
		return v[i];
	}

//if contains a element
	int member(const T&) const; // question?: is int enough?

//compare
	inline bool operator==(const mGArray<T>& a)
	{
		return v == a.v;
	}
	inline const bool operator==(const mGArray<T>& a) const
	{
		return v == a.v;
	}
	inline bool operator<(const mGArray<T>& a)
	{
		return v < a.v;
	}
	inline const bool operator<(const mGArray<T>& a) const
	{
		return v < a.v;
	}
	inline bool operator>(const mGArray<T>& a)
	{
		return v > a.v;
	}
	inline const bool operator>(const mGArray<T>& a) const
	{
		return v > a.v;
	}

//add to end
	mGArray<T>& operator+=(const T&);
	mGArray<T>& operator+=(const mGArray<T>&);

//find the first one and delete
	mGArray<T>& operator-=(const T&);

//remove all elements equal to parameter
	mGArray<T>& removeAll(const T&);

//heap sort
	void heapSort();

//help method ; question: static effects?
	static int compare(const void *, const void *);

	void quickSortWithQsort();

	void quickSortWithSort();

//find the intersection of two arrays
// the inner array's elements have the same relative order as they are in a1
	static void getIntersection(const mGArray<T>& a1, const mGArray<T>& a2,
			mGArray<T>& inter); // ugly ??  better way to do this?

//merg two soted array
//prorammer SHOULD make sure the two arrays are sorted
	mGArray<T>& mergeSortedArrays(mGArray<T>&);

	inline mGArray<T> swap(mGArray<T>& arr)
	{
		v.swap(arr.v);
		bool temp = sorted;
		sorted = arr.sorted;
		arr.sorted = temp;
		return *this;

	}

	//uqniue the array
	void unique_array();




};


} //namespace minigrail
#define _MGARRAY
#endif
