/*
    file: mGOstream.cpp
    Project: miniGrail
    License: LGPLv2.1 http://www.gnu.org/licenses/lgpl-2.1.html

    Created by: by Rui Zhou
    Maintainer: Cezar Campeanu <ccampeanu@upei.ca>

    Modified by Brandon Stewart July 2015
	  - Added namespace minigrail


  Ostream.cpp


  Created by Ray Rui Zhou on 11-10-09.

  Modified by Cezar Campeanu<ccampeanu@upei.ca> 2012-07-11
*/

#include <iostream>

namespace minigrail
{

//mGState

inline ostream& operator<<(ostream& os, const mGState& s)
{
	return os << (int) s.getNumber();
}
//mGTransition

inline ostream& operator<<(ostream& os, const mGTransition& t)
{

	os << (mGState) t.getStart();
	os << (char) ' ';
	os << indexToString((int) t.getLabel());

	os << (char) ' ';
	os << (mGState) t.getEnd();

	//std::cout << std::endl << (char)(t.getLabel() + 97) << std::endl;
	return os;
}

//mc DFA
inline ostream& operator<<(ostream& os, const mGNFA& d)
{ //start states
	for (int a = 0; a < d.getStartStates().size(); a++)
	{
		os << "(START) |- ";
		os << (d.getStartStates())[a];
		os << std::endl;
	}
	//transitions
	for (int a = 0; a < d.getTransitions().size(); a++)
	{
		os << d[a];
		os << std::endl;
	}
	//final states
	for (int a = 0; a < d.getFinalStates().size(); a++)
	{
		os << (d.getFinalStates())[a];
		os << " -| (FINAL)";
		os << std::endl;
	}

	return os;
}

//mc DFCA
inline ostream& operator<<(ostream& os, const mGDFCA& d)
{

	os << "l=" << d.getL() << endl;
	//start states
	for (int a = 0; a < d.getStartStates().size(); a++)
	{
		os << "(START) |- ";
		os << (d.getStartStates())[a];
		os << std::endl;
	}
	//transitions
	for (int a = 0; a < d.getTransitions().size(); a++)
	{
		os << d[a];
		os << std::endl;
	}
	//final states
	for (int a = 0; a < d.getFinalStates().size(); a++)
	{
		os << (d.getFinalStates())[a];
		os << " -| (FINAL)";
		os << std::endl;
	}

	return os;
}

//mGString

inline ostream& operator<<(ostream& os, const mGString& s)
{

	for (int a = 0; a < s.size(); a++)
	{
		os << indexToString(s[a]) << " ";

	}

	return os;
}

//mGLanguage

inline ostream& operator<<(ostream& os, const mGLanguage& l)
{

	for (int a = 0; a < l.size(); a++)
	{
		os << l[a] << std::endl;

	}

	return os;
}


} //namespace minigrail
