/*
*    Language Investigator and Stractural Analyzer (LISA) - 
*    A descriptive programming language
* LICENCE : LGPL
*
*
* FILE:			functions.h
*
* PURPOSE:		Pre-defined function declarations.
*
* CREATED AT:		December 16, 2011
*
* AUTHOR: 		Seyed Pooria Madani Kochak (s.p.madani@gmail.com)
* 
* Project Manager:	Dr. Cezar Campeanu (ccampeanu@upei.ca)
*/

/*
* LISA have some predefined functions. Each of those predefined functions
* are handled, type cheked in this file. Moreover, the appropriate intermediate
* code for each of functions is also get created by these functions.
* 
* Any new predefined function has to be added in this file and implemented in
* functions.c
* 
* Notes: All the function have same function's signiture.
* 
* Revision History
* --------------------------------------------------------------------------
* Seyed Pooria Madani - Creation							December 16, 2011
*/

#ifndef FUNCTIONS_H
#define FUNCTIONS_H
#include "symboltable.h"
#include "abstractsyntaxtree.h"



//Forward decleration of function in parser.y to report syntax erros.
void yyerror(char*);


/*
 * This function will generate code for returning
 * size of input parameter(ie. DFA variable).
 * The generated return value is INT.
 * It will only accept one parameter.
 */ 
unsigned char *size(int, struct Parameter_List *);
unsigned char *max(int, struct Parameter_List *);

/*
 * This function will generate code to 
 * read file that contains one of data 
 * types(DFA,NFA,String and etc).
 * The return value of generated code is depend on the type
 * first parameter (dfa,nfa,string and etc).
 * This function will accept two parameter:
 * 		(a) First parameter is type of content of the file
 * 		(b) String path of the file on local disk
 */
unsigned char *readfile(int, struct Parameter_List *);

/*
 * This function will generate code to write content of
 * a variable into file (DFA,NFA,String, and etc).
 * The generated code will return void.
 * This function only one paramter. The variable name.
 */
unsigned char *writefile(int, struct Parameter_List *);

/*
 * This function will generate code to print content of
 * it's paramter into standard output.
 * Most of the data types in the language are supported.
 * The generated code will return "void"
 */
unsigned char *print(int, struct Parameter_List *);

/*
 * This function will generate code to reverse
 * the content of the supplied data variable.
 * Only applicable to data types that support reverse order
 * (ie. DFA, NFA, REGEX)
 * The return value of generated code is the same 
 * type of supplied parameter.
 */
unsigned char *reverse(int, struct Parameter_List *);

/*
 * This function will generate code to compute
 * union of supplied parameters.
 * The parameters have to same type (DFA,NFA,REGEX),
 * and have to have the ability of union both.
 * The return value of generated code will have
 * the same type of the parameters.
 */
unsigned char *union_(int, struct Parameter_List *);

unsigned char *isfinite(int, struct Parameter_List *);
unsigned char *iscomplete(int, struct Parameter_List *);
unsigned char *isuniversal(int, struct Parameter_List *);
unsigned char *isdeterministic(int, struct Parameter_List *);

unsigned char *parse(int, struct Parameter_List *);
unsigned char *complete(int, struct Parameter_List *);


unsigned char *plus(int, struct Parameter_List *);
unsigned char *star(int, struct Parameter_List *);
unsigned char *reduce(int, struct Parameter_List *);
unsigned char *reachable(int, struct Parameter_List *);

unsigned char *shuffle(int, struct Parameter_List *);
unsigned char *cross(int, struct Parameter_List *);
unsigned char *nfatodfa(int, struct Parameter_List *);
unsigned char *dfatonfa(int, struct Parameter_List *);
unsigned char *concat(int, struct Parameter_List *);

#endif
