/*
* FILE:					icg.c
*    Language Investigator and Structural Analyzer (LISA) -
*    A descriptive programming language
* LICENCE :      LGPL *
*
* PURPOSE:	Intermidiate Code Generator (ICG) implementations.
*
* CREATED AT:	November 15, 2011
*
* AUTHOR: 	Seyed Pooria Madani Kochak (s.p.madani@gmail.com)
*
* Project Manager:	Dr. Cezar Campeanu (ccampeanu@upei.ca)
*/

/*
* This file holds implementation for Intermediate Code Generator.
* All the intermediate code for each grammar production is controlled by
* this file.
*
* Revision History
* --------------------------------------------------------------------------
* Seyed Pooria Madani - Creation							November 15, 2011
* Brandon Stewart     - Namespace support, and restructuring, details in
*                     "lisa-changelist.txt".                      June/July 2015
*/


#include "icg.h"
#include "symboltable.h"
#include "stack.h"

//original declaration in symboltable.h
extern Stack* namespaceStack;// Holds the namespaces that follow the "using ..." statement
					  //The namespace at the top of the stack takes priority


Stack* generatorStack;

int genTotal = 0; //totalNumber of generators
int heapCount = 0;

//Store all the heap pointer before
//assignment statement that needs to be deleted
int heapStack[50];
int heapStackCount= 0; //Keep the count;

//this should be removed in a future version.  Its used to deal with "constant_next_icg".
int gen_count = 0;


extern long codeSize;
extern unsigned char *code;
void init_icg(void) {
    templatefile = fopen("./lib/template.ccx","r");
    generatedfile = fopen("generated.cc","w");
    fseek (templatefile , 0 , SEEK_END);
    codeSize = ftell(templatefile) + 1;
    fseek (templatefile , 0 , SEEK_SET);
    code = (unsigned char *) malloc(sizeof(unsigned char) * codeSize);
    //rewind(templatefile);
    fread (code,1,codeSize,templatefile);

}

void declaration_icg(int index) {

    symbol_node node = symbol_table[index];	 //We read the symbol from the table

	int size = strlen(node.value) + strlen(node.value_name) + 1 + 1 + 2 + 9;

    unsigned char declarestmt[size];	//The size of the C++ code will not be bigger tha 100 character
    memset(declarestmt, '\0' , size * sizeof(unsigned char)); //Set all to EOF; it will be needed for strcat

	//This means the type is array
	if(node.array_capacity != 0)
    {
        snprintf(declarestmt, size, "%s %s[%d];\n\t", node.value, node.value_name, node.array_capacity);
	}
    else if (node.token_type == TOKENTYPE_STRINGLITERALS)
    {
        size += strlen(node.value) + 2;
        //TODO: replace "std::string " with non hard-coded value
        snprintf(declarestmt, size, "std::string %s(%s);\n\t", node.value_name, node.value);
    }
    else
    {
        snprintf(declarestmt, size, "%s %s;\n\t", node.value, node.value_name);
    }

    //At the end we have generated something like: "int x;"

    long declaresize = strlen(declarestmt) ;

    //Increase the code size by the size of the code generated here
    codeSize += declaresize ;

    //Create a temoporary string to the size of the code
    unsigned char new_code[codeSize];
    memset(new_code, '\0' , codeSize);

    //Find the $$Declaration$$ marker point in the code
    unsigned char *pointer = strstr(code,"$$Declaration$$");


    //Get the character index right before $$Declaration$$ marker
    long dexindex = (pointer - code);


    /*
     * The template code gets devided into two parts:
     *
     * 1) Part one, is from the beggining of the file till before the "$$Declaration$$" Marker
     * 2) Part two, is from the beggining of the "$$Declaration$$" Marker to end of the file.
     *
     * The new generated statement "declarestmt" should get inserted between these two parts.
     * Thus, We copy the first part to the new buffer. Then we insert the declared statement.
     * Finally, we concatinate the part two.
     *
     * The following code perform this operation
     */
    //Insert the code from begginig till before the $$Declaration$$ marker
    memcpy(new_code,code,dexindex);
    //printf("DECL: %s\n", code);
    //Insert the new generated code
    memcpy(&(new_code[dexindex]),declarestmt,declaresize);

    //Insert the rest of code from $$Declaration$$ to the end of file
    memcpy(&(new_code[dexindex+declaresize]),&(code[dexindex]),codeSize-(declaresize + dexindex));


    code = (unsigned char *) realloc(code,sizeof(unsigned char) * codeSize);

    memcpy(code,new_code,codeSize);

    //Print the new generated code
    // Only for debugging purposes
    //printf("\n%s\n",new_code);

}



//same as above, but deals with "grail::dfa" like syntax
void namespace_declaration_icg(int namespaceIndex, int operatorIndex, int typeIndex)
{

        symbol_node node = symbol_table[typeIndex];	 //We read the symbol from the table

        symbol_node namespaceNode = symbol_table[namespaceIndex];  //get the namespace name

        symbol_node operatorNode = symbol_table[operatorIndex];  //get the namespace operator "::"

    	int size = strlen(namespaceNode.value_name) + strlen(node.value) + strlen(node.value_name) + 1 + 1 + 2 + 9;

    	if(node.token_type == TOKENTYPE_STRINGLITERALS){
    			size += strlen(node.value) + 2;
    	}

        unsigned char declarestmt[size];	//The size of the C++ code will not be bigger tha 100 character
        memset(declarestmt, '\0' , size * sizeof(unsigned char)); //Set all to EOF; it will be needed for strcat

        //This means the type is array
    	if(node.array_capacity!=0){ //grail::dfa[3] myDFA;

            snprintf(declarestmt, size, "%s%s%s %s[%d];\n\t",namespaceNode.value,
             operatorNode.value_name,
             node.value,
             node.value_name,
             node.array_capacity);
    	}
        else //"grail::dfa myDFA;"
        {
            snprintf(declarestmt, size, "%s%s%s %s;\n\t",namespaceNode.value,
             operatorNode.value_name,
             node.value,
             node.value_name);
        }


        //At the end we have generated something like: int <node.value_name> ;

        long declaresize = strlen(declarestmt) ;

        //Increase the code size by the size of the code generated here
        codeSize += declaresize ;


        //Create a temoporary string to the size of the code
        unsigned char new_code[codeSize];
        memset(new_code, '\0' , codeSize);

        //Find the $$Declaration$$ marker point in the code
        unsigned char *pointer = strstr(code,"$$Declaration$$");


        //Get the character index right before $$Declaration$$ marker
        long dexindex = (pointer - code);


        /*
         * The template code gets devided into two parts:
         *
         * 1) Part one, is from the beggining of the file till before the "$$Declaration$$" Marker
         * 2) Part two, is from the beggining of the "$$Declaration$$" Marker to end of the file.
         *
         * The new generated statement "declarestmt" should get inserted between these two parts.
         * Thus, We copy the first part to the new buffer. Then we insert the declared statement.
         * Finally, we concatinate the part two.
         *
         * The following code perform this operation
         */
        //Insert the code from begginig till before the $$Declaration$$ marker
        memcpy(new_code,code,dexindex);

        //Insert the new generated code
        memcpy(&(new_code[dexindex]),declarestmt,declaresize);

        //Insert the rest of code from $$Declaration$$ to the end of file
        memcpy(&(new_code[dexindex+declaresize]),&(code[dexindex]),codeSize-(declaresize + dexindex));



        code = (unsigned char *) realloc(code,sizeof(unsigned char) * codeSize);

        memcpy(code,new_code,codeSize);

        //Print the new generated code
        // Only for debugging purposes
        //printf("\n%s\n",new_code);

}




struct Constant *constant_next_icg(int index){
	struct Constant *return_const = (struct Constant *)malloc(sizeof(struct Constant));
	unsigned char *returnvalue =  NULL;
	symbol_node node = symbol_table[index];
	int size = 30;
    returnvalue = (unsigned char *) malloc(sizeof(unsigned char) * size);
    memset(returnvalue,'\0',size);


    sprintf(returnvalue,"gen%d.next()",generatorStack->data);

    return_const->type = node.node_type;
    return_const->sindex_constant = index;
    return_const->code = returnvalue;
    return return_const;


}
struct Constant *constant_icg(int index) {

    struct Constant *return_const = (struct Constant *)malloc(sizeof(struct Constant));

    unsigned char *returnvalue =  NULL;
    symbol_node node = symbol_table[index];
    int value;
    int size;
    switch (node.token_type) {
    case TOKENTYPE_NUMBER:
    	size = strlen(node.value_name)+1;
    	        returnvalue = (unsigned char *) malloc(sizeof(unsigned char) * size);
    	        memset(returnvalue,'\0',sizeof(unsigned char) * size);
    	        memcpy(returnvalue,node.value_name,size);
    	        /*
        returnvalue = (unsigned char *) malloc(sizeof(unsigned char) * 9);
        memset(returnvalue,'\0',sizeof(unsigned char) * 9);
        value = *(int *)node.value;
        sprintf(returnvalue,"%d ",value);
	*/
        //For debugging
        // printf("********************* %s\n",returnvalue);
        break;

    case TOKENTYPE_KEYWORD:
        size = strlen(node.value_name)+1;
        returnvalue = (unsigned char *) malloc(sizeof(unsigned char) * size);
        memset(returnvalue,'\0',sizeof(unsigned char) * size);
        memcpy(returnvalue,node.value_name,size);
        break;

    case TOKENTYPE_STRINGLITERALS:
        size = strlen(node.value_name)+1;
        returnvalue = (unsigned char *) malloc(sizeof(unsigned char) * size);
        memset(returnvalue,'\0',sizeof(unsigned char) * size);
        memcpy(returnvalue,node.value_name,size);

        break;

    case TOKENTYPE_ID:
        size = strlen(node.value_name)+1 + 11; //11 for array size mentioning - 9 for integer and 2 for '[' and ']'
        returnvalue = (unsigned char *) malloc(sizeof(unsigned char) * size);
        memset(returnvalue,'\0',sizeof(unsigned char) * size);
        memcpy(returnvalue,node.value_name,size);
        if(node.array_capacity != 0){
			//the variable is array item
			unsigned char *capacity = (unsigned char *) malloc(sizeof(unsigned char) * 9);
			memset(capacity,'\0',sizeof(unsigned char) * 9);
			sprintf(returnvalue,"%d ",value);
		}
        break;

    }
    return_const->type = node.node_type;
    return_const->sindex_constant = index;
    return_const->code = returnvalue;
    return return_const;
}

struct Constant *constant_wrap_function(struct Function *func) {

    struct Constant *return_const = (struct Constant *)malloc(sizeof(struct Constant));
    return_const->function = func;
    return_const->code = func->code;
    return_const->type = func->type;
    return return_const;
}

struct Constant *constant_wrap_variable(struct Variable *var) {

    struct Constant *return_const = (struct Constant *)malloc(sizeof(struct Constant));
    return_const->variable = var;
    return_const->code = var->code;
    return_const->type = var->type;
	return_const->sindex_constant = var->id_index;
    return return_const;
}


struct Factor *factor_icg(struct Simple_Expression *smpl_expr) {

    struct Factor *factor = (struct Factor *)malloc(sizeof(struct Factor));

    int size = strlen(smpl_expr->code) + 3; // 2 for parentheses and one for end of the line
    unsigned char *generated_stmt = (unsigned char *) malloc(sizeof(unsigned char) * size);
    memset(generated_stmt,'\0',size);
    generated_stmt[0] = '(';
    memcpy(&(generated_stmt[1]), smpl_expr->code, strlen(smpl_expr->code));
    generated_stmt[strlen(smpl_expr->code)+1] = ')';

    factor->type = smpl_expr->type;
    factor->simple_expression = smpl_expr;
    factor->code = generated_stmt;
    return factor;
}

struct Factor *factor_wrap_constant(struct Constant *constant) {
    struct Factor *factor = (struct Factor *)malloc(sizeof(struct Factor));
    factor->constant = constant;
    factor->code = constant->code;
    factor->type = constant->type;
    return factor;
}


struct Term	*term_wrap_factor(struct Factor *factor) {


    struct Term *term = (struct term *)malloc(sizeof(struct Term));
    term->factor = factor;
    term->code = factor->code;
    term->type = factor->type;

    return term;
}
struct Term *term_icg(struct Term *term,int oprIndex,struct Factor *factor) {

    struct Term *return_term = (struct Term*)malloc(sizeof(struct Term));

    /*
     * Type checking is neccessary, since not all the types in the language
     * supporting multiplication operators.
     * The only type that support multiplication in this language is "int"
     *
     * (1) First "TERM" and "FACTOR" needs to be checked and be made sure that
     * their type is INT.
     * 		(1.a) If is, then we set the return_term type to INT
     * 		(1.b) Otherwise, we throw syntax error regarding this matter.
     */

    if (term->type != NODETYPE_INTEGER || factor->type != NODETYPE_INTEGER)
        yyerror("Multiplication operators only can be used for INTEGER types.");

    return_term->type = NODETYPE_INTEGER;
    int size = strlen(term->code) + strlen(factor->code) + 3;
    unsigned char *generated_stmt = (unsigned char *) malloc(sizeof(unsigned char) * size);
    memset(generated_stmt,'\0',size);
    memcpy(generated_stmt, term->code, strlen(term->code));

    generated_stmt[strlen(term->code)] = ((unsigned char *)symbol_table[oprIndex].value_name)[0];
    memcpy(&(generated_stmt[strlen(term->code)+1]), factor->code, strlen(factor->code));

    return_term->term = term;
    return_term->factor = factor;
    return_term->index_multiplication_operator = oprIndex;
    return_term->code = generated_stmt;

    return return_term;

}
struct Add_Expression *addexpression_wrap_term(struct Term *term) {

    struct Add_Expression *add_expr = (struct Add_Expression *)malloc(sizeof(struct Add_Expression));
    add_expr->term = term;
    add_expr->code = term->code;
    //printf("IMhereeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee");
    add_expr->type = term->type;
    return add_expr;

}

//2015-10-20 Brandon Stewart:  changed "concat_code#" vars to not use pointers in the string
struct Add_Expression *addexpression_icg(struct Add_Expression *addexpression,int oprIndex,struct Term *term) {

    struct Add_Expression *add_expr = (struct Add_Expression *)malloc(sizeof(struct Add_Expression));
    /*
     * Type checking is neccessary, since not all the types in the language
     * supporting addition operators.
     * The only types that support multiplication in this language are "int" and "string"
     *
     * (1) First "addexpression" and "term" needs to be checked and be made sure that
     * their type is INT or STRING.
     * 		If is not, we throw syntax error regarding this matter.
     * (2) String only support "+" operator and not "-", therefore, for string we have to
     * 	check ferther and make sure the operator has been used properly.
     */

    if ( (addexpression->type != NODETYPE_INTEGER && addexpression->type != NODETYPE_STRING) ||
            (term->type != NODETYPE_INTEGER && term->type != NODETYPE_STRING) ) {
        yyerror("Addition operators only can be used for STRING and INT types.");


    }

    symbol_node node = symbol_table[oprIndex];
    if ( (addexpression->type == NODETYPE_STRING) || (term->type == NODETYPE_STRING) ) {

        if ( ((unsigned char *)node.value_name)[0] == '-') {

            yyerror("'-' can not be used between STRING and INT.");
        } else {
            /*
             * If one is at least string and the operator is not '-', then the result
             * type is definitly string and is string concatination
             */

			unsigned char *concat_code1 = "LISASTD::string_concat(%s,LISASTD::int_to_string(%s))\0";
			unsigned char *concat_code2 = "LISASTD::string_concat(%s,%s)\0";
			unsigned char *concat_code3 = "LISASTD::string_concat(LISASTD::int_to_string(%s),%s)\0";
			unsigned char *generated_stmt;
			add_expr->type = NODETYPE_STRING;
			if(addexpression->type == term->type){

				int size = strlen(concat_code2) + strlen(addexpression->code) + strlen(term->code) + 5; // <<
				generated_stmt = (unsigned char *) malloc(sizeof(unsigned char) * size);
				memset(generated_stmt,'\0',size);
				sprintf(generated_stmt,concat_code2,addexpression->code,term->code);
			}else if(addexpression->type == NODETYPE_STRING){

				int size = strlen(concat_code1) + strlen(addexpression->code) + strlen(term->code) + 5; // <<
				generated_stmt = (unsigned char *) malloc(sizeof(unsigned char) * size);
				memset(generated_stmt,'\0',size);
				sprintf(generated_stmt,concat_code1,addexpression->code,term->code);
			}else if(addexpression->type == NODETYPE_INTEGER){

				int size = strlen(concat_code3) + strlen(addexpression->code) + strlen(term->code) + 5; // <<
				generated_stmt = (unsigned char *) malloc(sizeof(unsigned char) * size);
				memset(generated_stmt,'\0',size);
				sprintf(generated_stmt,concat_code3,addexpression->code,term->code);
			}else{
				yyerror("Syntax error on string concatination");
			}
			add_expr->code = generated_stmt;
        }
    } else {
        /*
         * However, none is string, we are sure the result is INT
         */
        add_expr->type = NODETYPE_INTEGER;
		int size = strlen(addexpression->code) + strlen(term->code) + 3;
		unsigned char *generated_stmt = (unsigned char *) malloc(sizeof(unsigned char) * size);
		memset(generated_stmt,'\0',size);

		memcpy(generated_stmt, addexpression->code, strlen(addexpression->code));

		generated_stmt[strlen(addexpression->code)] = ((unsigned char *)symbol_table[oprIndex].value_name)[0];
		memcpy(&(generated_stmt[strlen(addexpression->code)+1]), term->code, strlen(term->code));

		add_expr->code = generated_stmt;
	}

    add_expr->term = term;
    add_expr->add_expression = addexpression;
    add_expr->index_add_operator = oprIndex;
    return add_expr;


}


struct Relation_Expression *relationexpression_wrap_addexpression(struct Add_Expression *add_expr) {

    struct Relation_Expression *rel_expr =
        (struct Relation_Expression *)malloc(sizeof(struct Relation_Expression));


    rel_expr->code = add_expr->code;
    rel_expr->add_expression1 = add_expr;
    rel_expr->type = add_expr->type;

    return rel_expr;


}
struct Relation_Expression *relationexpression_icg(struct Add_Expression *addexpression1,
        int oprIndex,struct Add_Expression *addexpression2) {

    struct Relation_Expression *rel_expr =
        (struct Relation_Expression *)malloc(sizeof(struct Relation_Expression));

    /*
     * Type checking:
     * The supported relational operators are as fallow:
     * > < <= >= : INTEGER and STRING
     * == != : INTEGER and STRING and BOOL
     *
     * Therefore, check needs to be done for all the two category to make sure
     * correct use. (Good type checking prevent hours of debugging.)
     *
     * * The result always is boolean type. *
     */

    rel_expr->type = NODETYPE_BOOLEAN;

    if ( (addexpression1->type != NODETYPE_BOOLEAN && addexpression1->type != NODETYPE_INTEGER && addexpression1->type != NODETYPE_STRING) &&
            (addexpression2->type != NODETYPE_BOOLEAN && addexpression2->type != NODETYPE_INTEGER && addexpression2->type != NODETYPE_STRING) ) {

        yyerror("Relational operators only supported for STRING, INTEGER, and BOOLEAN data types.");
    }

    symbol_node node = symbol_table[oprIndex];

    if (strcmp(node.value_name,"==") != 0 || strcmp(node.value_name,"!=") != 0) {
        if (addexpression1->type == NODETYPE_BOOLEAN || addexpression2->type == NODETYPE_BOOLEAN) {
            yyerror("'<', '>' are not supported for BOOLEAN data type.");
        }
    }

    /*
     * Then we have to make sure the both side are the same type
     */
    if (addexpression1->type != addexpression2->type) {
        yyerror("Both operatands on the relational expression have to have same type.");
    }

    int size = strlen(addexpression1->code) + strlen(addexpression2->code) + 4;

    unsigned char *generated_stmt = (unsigned char *) malloc(sizeof(unsigned char) * size);
    memset(generated_stmt,'\0',size);

    memcpy(generated_stmt, addexpression1->code, strlen(addexpression1->code));

    memcpy(&(generated_stmt[strlen(addexpression1->code)]), symbol_table[oprIndex].value_name, strlen(symbol_table[oprIndex].value_name));

    memcpy(&(generated_stmt[strlen(addexpression1->code)+ strlen(symbol_table[oprIndex].value_name)]),
           addexpression2->code, strlen(addexpression2->code));

    rel_expr->code = generated_stmt;
    rel_expr->add_expression1 = addexpression1;
    rel_expr->add_expression2 = addexpression2;
    rel_expr->index_relation_operator = oprIndex;
    return rel_expr;

}

struct Unary_Relation_Expression *unaryrelationexpression_wrap_relationexpression(struct Relation_Expression *relationexpression) {

    struct Unary_Relation_Expression *ur_expr = (struct Unary_Relation_Expression*)
            malloc(sizeof(struct Unary_Relation_Expression));
    ur_expr->relation_expression = relationexpression;
    ur_expr->code = relationexpression->code;
    ur_expr->type = relationexpression->type;


    return ur_expr;
}

struct Unary_Relation_Expression *unaryrelationexpression(struct Unary_Relation_Expression *relationexpression) {

    struct Unary_Relation_Expression *ur_expr = (struct Unary_Relation_Expression*)
            malloc(sizeof(struct Unary_Relation_Expression));

    /*
     * Type Checking:
     * parameter type has to be boolean, and the result type is boolean
     */
    ur_expr->type =  NODETYPE_BOOLEAN;

    if (relationexpression->type !=  NODETYPE_BOOLEAN) {
        yyerror("Unary-relationexpression has to be boolean type.");
    }

    int size = strlen(relationexpression->code) + 3;

    unsigned char *generated_stmt = (unsigned char *) malloc(sizeof(unsigned char) * size);
    memset(generated_stmt,'\0',size);
    generated_stmt[0] = '!';
    memcpy(&(generated_stmt[1]), relationexpression->code, strlen(relationexpression->code));
    //memcpy( generated_stmt , relationexpression, strlen(relationexpression));
    ur_expr->unary_relation_expression = relationexpression;
    ur_expr->code = generated_stmt;
    return ur_expr;


}


struct Or_Expression *orexpression_wrap_unaryrelationexpression(struct Unary_Relation_Expression *unaryrelationexpression) {

    struct Or_Expression *or_expr = (struct Or_Expression *)malloc(sizeof(struct Or_Expression));



    or_expr->code = unaryrelationexpression->code;
    or_expr->unary_relation_expression = unaryrelationexpression;
    or_expr->type = unaryrelationexpression->type;
    return or_expr;

}

struct Or_Expression *orexpression_icg(struct Or_Expression *orexpression,int oprIndex,
                                       struct Unary_Relation_Expression *unaryrelationexpression) {

    struct Or_Expression *or_expr = (struct Or_Expression *)malloc(sizeof(struct Or_Expression));



    /*
     * Type Checking:
     * parameters type have to be boolean, and the result type is boolean
     */
    or_expr->type = NODETYPE_BOOLEAN;
    if (unaryrelationexpression->type !=  NODETYPE_BOOLEAN ||
            orexpression->type !=  NODETYPE_BOOLEAN) {
        yyerror("AND-relationexpression has to be boolean type.");
    }


    int size = strlen(orexpression->code) + strlen(unaryrelationexpression->code)
               +  strlen(symbol_table[oprIndex].value_name) + 2;

    unsigned char *generated_stmt = (unsigned char *) malloc(sizeof(unsigned char) * size);
    memset(generated_stmt,'\0',size);

    memcpy(generated_stmt, orexpression->code, strlen(orexpression->code));

    memcpy(&(generated_stmt[strlen(orexpression->code)]), symbol_table[oprIndex].value_name, strlen(symbol_table[oprIndex].value_name));

    memcpy(&(generated_stmt[strlen(orexpression->code)+ strlen(symbol_table[oprIndex].value_name)]),
           unaryrelationexpression->code, strlen(unaryrelationexpression->code));

    or_expr->code = generated_stmt;
    or_expr->or_expression = orexpression;
    or_expr->unary_relation_expression = unaryrelationexpression;
    or_expr->index_operator = oprIndex;

    return or_expr;


}

struct Simple_Expression *simpleexpression_icg(struct Simple_Expression *simple_expr,int oprIndex
        ,struct Or_Expression *or_expr) {

    struct Simple_Expression *smpl_expr = (struct Simple_Expression *)malloc(sizeof(struct Simple_Expression));




    /*
     * Type Checking:
     * parameters type have to be boolean, and the result type is boolean
     */
    smpl_expr->type = NODETYPE_BOOLEAN;
    if (simple_expr->type !=  NODETYPE_BOOLEAN ||
            or_expr->type !=  NODETYPE_BOOLEAN) {
        yyerror("OR-relationexpression has to be boolean type.");
    }


    int size = strlen(simple_expr->code) + strlen(or_expr->code)
               +  strlen(symbol_table[oprIndex].value_name) + 2;

    unsigned char *generated_stmt = (unsigned char *) malloc(sizeof(unsigned char) * size);
    memset(generated_stmt,'\0',size);

    memcpy(generated_stmt, simple_expr->code, strlen(simple_expr->code));

    memcpy(&(generated_stmt[strlen(simple_expr->code)]), symbol_table[oprIndex].value_name, strlen(symbol_table[oprIndex].value_name));

    memcpy(&(generated_stmt[strlen(simple_expr->code)+ strlen(symbol_table[oprIndex].value_name)]),
           or_expr->code, strlen(or_expr->code));

    smpl_expr->code = generated_stmt;
    smpl_expr->or_expression = or_expr;
    smpl_expr->sm_expression = simple_expr;
    smpl_expr->index_operator = oprIndex;

    return smpl_expr;
}

struct Simple_Expression *simpleexpression_wrap_orexpression(struct Or_Expression *or_expr) {


    struct Simple_Expression *smpl_expr = (struct Simple_Expression *)malloc(sizeof(struct Simple_Expression));
    smpl_expr->code = or_expr->code;
    smpl_expr->or_expression = or_expr;
    smpl_expr->type = or_expr->type;
    return smpl_expr;

}


struct Expression *expression_icg(struct Variable *variable,int relopSIndex,struct Expression *inputexpr) {

    struct Expression *expr = (struct Expression *)malloc(sizeof(struct Expression));

    /*
     * '=' for INT, DFA, NFA, REGEX, BOOLEAN
     * '+=' '*=' '/=' '-=' for INT only
     *
     * Type check:
     * '=' , both side of assignment operator has to be same type.
     * for other operators, both side has to be INT.
     */

	int variableSIndex = variable->id_index;

    symbol_node variableNode = symbol_table[variableSIndex];
    symbol_node relopNode = symbol_table[relopSIndex];



    if (strcmp(relopNode.value_name,"=") != 0) {
        if (variableNode.node_type !=  NODETYPE_INTEGER || inputexpr->type != NODETYPE_INTEGER) {
            yyerror("The assignment operator being used, is only for integer data types.");
        }
    } else {
        if (variableNode.node_type != inputexpr->type  ) {
            yyerror("Both sides of the assignment has to be the same type.");
        }
    }

    //int length_stmt = strlen(inputexpr->code) + strlen(variableNode.value_name) + strlen(relopNode.value_name) + 2;
	int length_stmt = strlen(inputexpr->code) + strlen(variable->code) + strlen(relopNode.value_name) + 2;

    //unsigned char generated_stmt[length_stmt];
    unsigned char *generated_stmt = (unsigned char *) malloc(sizeof(unsigned char) * length_stmt);
    memset(generated_stmt,'\0',length_stmt);

    strcat(generated_stmt,variable->code);
    strcat(generated_stmt,relopNode.value_name);
    strcat(generated_stmt,inputexpr->code);




    expr->code = generated_stmt;
    expr->expression= inputexpr;
    expr->variable_index = variableSIndex;
    expr->operator_index = relopSIndex;
    expr->type = variableNode.node_type;


    return expr;
    //print_statement(generated_stmt);

}

struct Expression *expression_wrap_simpleexpression_icg(struct Simple_Expression *smpl) {
    struct Expression *expr = (struct Expression *)malloc(sizeof(struct Expression));


    expr->sm_expression = smpl;
    expr->code = smpl->code;
    expr->type = smpl->type;

    return expr;


}


struct Variable *variable_icg(int idIndex, int arrayIndex) {


    /*
     * Check to see if the variable is declared before is being used?
     */
    symbol_node id = symbol_table[idIndex];
	symbol_node index = symbol_table[arrayIndex];
    if (id.node_type == NODETYPE_NULL) {
        /*
         * Should not use strcat, since strcat will change value_name of id in
         * symbol table. However, since at this point the program will get terminitated
         * shouldn't be a problem.
         */
        unsigned char msg[] =" is not declared before.\0";
        int msgsize = strlen(id.value_name) + strlen(msg) + 1;
        unsigned char *error = (unsigned char*)malloc(sizeof(unsigned char) * msgsize);
        memset(error,'\0',msgsize);

        strncpy(error,id.value_name,strlen(id.value_name));
        strncpy(&(error[strlen(id.value_name)]),msg,strlen(msg));
        yyerror(error);
    }



	int i;

	struct Variable *var = (struct Variable *)malloc(sizeof(struct Variable));


	int size = strlen(id.value_name)+1 + 11 + (arrayIndex >= 0? strlen(index.value_name) : 0);
	unsigned char *generated_code = (unsigned char *) malloc(sizeof(unsigned char) * size);
    memset(generated_code,'\0',sizeof(unsigned char) * size);
    memcpy(generated_code,id.value_name,size);

	if( arrayIndex >= 0){
			if(id.array_capacity <=0){
				yyerror("You are trying to use a non-array variable as a array one.");
			}
			if(index.token_type == TOKENTYPE_NUMBER && index.node_type == NODETYPE_INTEGER){
				int cap = *((int *)index.value);
				if(cap  >= id.array_capacity){
					yyerror("Trying access array out of bound");
				}
				strcat(generated_code,"[");
				unsigned char *capacity = (unsigned char *)malloc(sizeof(unsigned char) * 9);
				sprintf(capacity,"%d",cap);
				strcat(generated_code,capacity);
				strcat(generated_code,"]");
			}else if(index.token_type == TOKENTYPE_ID && index.node_type == NODETYPE_INTEGER){
				strcat(generated_code,"[");
				strcat(generated_code,index.value_name);
				strcat(generated_code,"]");
			}else{
				yyerror("Index number for the array is wrong.");
			}

	}

	var->code = generated_code;
	var->type = id.node_type;
	var->id_index=idIndex;


	return var;

}


unsigned char *compundstatement_icg(unsigned char *statements) {

    int size = strlen(statements) + 3;

    unsigned char *generated_stmt = (unsigned char *) malloc(sizeof(unsigned char) * size);
    memset(generated_stmt,'\0',size);
    generated_stmt[0] = '{';
    memcpy(&(generated_stmt[1]), statements, strlen(statements));
    generated_stmt[strlen(statements) + 1] = '}';

    return generated_stmt;

}


unsigned char *statements_icg(unsigned char *stmts, unsigned char *stmt) {
    int size = strlen(stmts) + strlen(stmt) +  2;


    unsigned char *generated_stmt = (unsigned char *) malloc(sizeof(unsigned char) * size);
    memset(generated_stmt,'\0',size);

    memcpy(generated_stmt, stmts, strlen(stmts));

    memcpy(&(generated_stmt[strlen(stmts)]),stmt, strlen(stmt));

    return generated_stmt;
}




unsigned char *whilestatement_icg(struct Expression *expression, unsigned char *statement) {

	/*
	 * Check to see if the Expression is a boolean expression
	 *
	 */

	 if(expression->type != NODETYPE_BOOLEAN){

		 yyerror("Boolean expression has to be used for while statement.");
	 }

    //while ( expression ) statement
    int size = strlen(expression->code) + strlen(statement) +  8;


    unsigned char *generated_stmt = (unsigned char *) malloc(sizeof(unsigned char) * size);
    memset(generated_stmt,'\0',size);

    memcpy(generated_stmt, "while(" , 6 );
    memcpy(&(generated_stmt[6]), expression->code, strlen(expression->code));
    generated_stmt[ 6 + strlen(expression->code)] = ')';
    memcpy(&(generated_stmt[6 + strlen(expression->code) + 1]),statement, strlen(statement));

    return generated_stmt;
}

unsigned char *getGeneratorName(){
	unsigned char *str = (unsigned char *) malloc(sizeof(unsigned char) * 30);
	memset(str,'\0',30);

//    sleep(1);
	sprintf(str,"gen%d",generatorStack->data);
	return str;
}

//NOT IN USE AS OF Lisa-0.2.0.  Considered for removal.
unsigned char *getHeapObjectName(){
	unsigned char *str = (unsigned char *) malloc(sizeof(unsigned char) * 30);
	memset(str,'\0',30);
	sprintf(str,"heap%d",heapCount++);
	heapStack[heapStackCount++] = add_finitemachine_pointer_symbol_node(str);

	return str;
}

void addGeneratorToStack()
{
    StackPush(&generatorStack, genTotal++);
}


unsigned char *generatingstatement_icg(int typeIndex,int nIndex,int kIndex,unsigned char *statement){



	symbol_node type = symbol_table[typeIndex];
	symbol_node n = symbol_table[nIndex];
	symbol_node k = symbol_table[kIndex];
	/*
	 * Template:
	 *	Generator(9) gen(n,k,Generator::SEQUENTIAL);\0 {size of this statement is 50 + n and k sizes
	 *	while(gen.hasNext())statement
	 *
	 */

     //the global namespace does not contain the generate statement, so check the stack

     Stack* tempStack = namespaceStack;
     symbol_node namespace;
     symbol_node generatorNode;
     int generatorIndex;
     int found = 0; //end the search when the generator node is determined
     int multipleDefinitions = 0; //checks to make sure whether (namespace)::generator is needed
     if (StackIsEmpty(tempStack))
     {
         yyerror("unknown type: \"generator\"\n");
     }
     else
     {
         while (!StackIsEmpty(tempStack) && !found)
         {
             namespace =  symbol_table[StackPop(&tempStack)];
             symbol_node compareNode = namespace;
             compareNode.token_type = TOKENTYPE_KEYWORD;
             compareNode.value_name = "generate"; //TODO: should be changed to non-hardcoded
             compareNode.node_type = NODETYPE_NULL;


            //check if "generate" is in namespace
            if ((generatorIndex = find_symbol_node(&compareNode)) >= 0)
            {
                // it is, check to see if another namespace has "generate", to see
                // if (namespace)::Generator syntax is needed.
                found = 1;
                generatorNode = symbol_table[generatorIndex];
                if (count_symbols_same_declaration(&generatorNode))
                {
                    multipleDefinitions = 1;
                }

            }

         }

     }

     if (n.node_type != NODETYPE_INTEGER)
     {
         int size = strlen("variable \"\" in ::generator expected to be of type \"int\"") + strlen(namespace.value_name) + strlen(n.value_name) + 1; //+1 for null
         char errorMsg[size];
         snprintf(errorMsg, size, "variable \"%s\" in %s::generator expected to be of type \"int\"", n.value_name, namespace.value_name);
         yyerror(errorMsg);
     }

     if (k.node_type != NODETYPE_INTEGER)
     {
         int size = strlen("variable \"\" in ::generator expected to be of type \"int\"") + strlen(namespace.value_name) + strlen(k.value_name) + 1; //+1 for null
         char errorMsg[size];
         snprintf(errorMsg, size, "variable \"%s\" in %s::generator expected to be of type \"int\"", k.value_name, namespace.value_name);
         yyerror(errorMsg);
     }

    char *generatorName = getGeneratorName();

	int statement_size = strlen(statement) + 50;
	unsigned char *sub_stmt = (unsigned char *)malloc(sizeof(unsigned char) * statement_size);
	memset(sub_stmt,'\0',statement_size);
	statement[0] = ' ';


	sprintf(sub_stmt,"{ %s",statement);


    unsigned char *stmt;

    if (found)
    {
        if (multipleDefinitions)
        {
            int size = 100 + strlen(namespace.value) + strlen(n.value_name) + strlen(k.value_name)+ strlen(type.value_name)+ (3 * strlen(generatorName)) + strlen(sub_stmt) + 3;
        	stmt = (unsigned char *)malloc(sizeof(unsigned char) * size);
        	memset(stmt,"\0",size);
        	if(strcmp(type.value_name,"enumerate") == 0)
        		sprintf(stmt,"%s::%s %s(%s,%s, %s::%s::SEQUENTIAL); \
        					\nwhile(%s.hasNext())%s",namespace.value, generatorNode.value, generatorName,n.value_name,k.value_name, namespace.value, generatorNode.value,generatorName,sub_stmt);
        	else if(strcmp(type.value_name,"random") == 0)
        		sprintf(stmt,"%s::%s %s(%s,%s, %s::%s::RANDOM); \
                            \nwhile(%s.hasNext())%s",namespace.value, generatorNode.value,generatorName,n.value_name,k.value_name, namespace.value, generatorNode.value,generatorName,sub_stmt);
        }
        else //if only one namespace has a generator
        {
            int size = 100 + strlen(n.value_name) + strlen(k.value_name)+ strlen(type.value_name)+ (3 * strlen(generatorName)) + strlen(sub_stmt) + 3;
        	stmt = (unsigned char *)malloc(sizeof(unsigned char) * size);
        	memset(stmt,"\0",size);
        	if(strcmp(type.value_name,"enumerate") == 0)
        		sprintf(stmt,"%s %s(%s,%s, %s::SEQUENTIAL); \
        					\nwhile(%s.hasNext())%s",generatorNode.value,generatorName,n.value_name,k.value_name, generatorNode.value,generatorName,sub_stmt);
        	else if(strcmp(type.value_name,"random") == 0)
        		sprintf(stmt,"%s %s(%s,%s, %s::RANDOM); \
                            \nwhile(%s.hasNext())%s",generatorNode.value,generatorName,n.value_name,k.value_name, generatorNode.value,generatorName,sub_stmt);
        }

    }
    else
    {
        yyerror("unknown symbol \"generate\"");
    }

    StackPop(&generatorStack);

	return stmt;
}


unsigned char *namespace_generatingstatement_icg(int namespaceIndex,int typeIndex,int nIndex,int kIndex,unsigned char *statement){


	symbol_node type = symbol_table[typeIndex];
	symbol_node n = symbol_table[nIndex];
	symbol_node k = symbol_table[kIndex];
    symbol_node namespace = symbol_table[namespaceIndex];

    //check if the namespace exists
    if (namespace.token_type != TOKENTYPE_NAMESPACE)
    {
        int size = strlen("Unknown namespace \"\"") + strlen(namespace.value_name) + 1; //+1 for null
        char errorMsg[size];
        snprintf(errorMsg, size, "Unknown namespace \"%s\"", namespace.value_name);
        yyerror(errorMsg);
    }

    //make sure the parameters are integers
    if (n.node_type != NODETYPE_INTEGER)
    {
        int size = strlen("variable \"\" in ::generator expected to be of type \"int\"") + strlen(namespace.value_name) + strlen(n.value_name) + 1; //+1 for null
        char errorMsg[size];
        snprintf(errorMsg, size, "variable \"%s\" in %s::generator expected to be of type \"int\"", n.value_name, namespace.value_name);
        yyerror(errorMsg);
    }

    if (k.node_type != NODETYPE_INTEGER)
    {
        int size = strlen("variable \"\" in ::generator expected to be of type \"int\"") + strlen(namespace.value_name) + strlen(k.value_name) + 1; //+1 for null
        char errorMsg[size];
        snprintf(errorMsg, size, "variable \"%s\" in %s::generator expected to be of type \"int\"", k.value_name, namespace.value_name);
        yyerror(errorMsg);
    }


    char *generatorName = getGeneratorName();

	int statement_size = strlen(statement) + 50;
	unsigned char *sub_stmt = (unsigned char *)malloc(sizeof(unsigned char) * statement_size);
	memset(sub_stmt,'\0',statement_size);
	statement[0] = ' ';


	sprintf(sub_stmt,"{ %s",statement);
	int size = 100 + strlen(n.value_name) + strlen(k.value_name)+ strlen(type.value_name)+ (3 * strlen(generatorName)) + strlen(sub_stmt) + 3;
	unsigned char *stmt = (unsigned char *)malloc(sizeof(unsigned char) * size);
	memset(stmt,"\0",size);
	if(strcmp(type.value_name,"enumerate") == 0)
		sprintf(stmt,"%s::Generator %s(%s,%s, %s::Generator::SEQUENTIAL); \
					\nwhile(%s.hasNext())%s",namespace.value,generatorName,n.value_name,k.value_name, namespace.value,generatorName,sub_stmt);
	else if(strcmp(type.value_name,"random") == 0)
		sprintf(stmt,"%s::Generator %s(%s,%s, %s::Generator::RANDOM); \
                    \nwhile(%s.hasNext())%s",namespace.value,generatorName,n.value_name,k.value_name, namespace.value,generatorName,sub_stmt);

    StackPop(&generatorStack);

	return stmt;
}


unsigned char *expressionstatement_icg(unsigned char *expression) {


    //expression ;
    int size = strlen(expression) + 4;
    for(int i =0; i < heapStackCount; i++){
    	symbol_node node = symbol_table[heapStack[i]];
    	size += strlen(  ((unsigned char *)(node.value_name))    ) + 9; //delete <name>;
    }
   // printf("I'mhehehehe\n");

    unsigned char *generated_stmt = (unsigned char *) malloc(sizeof(unsigned char) * size);
    memset(generated_stmt,'\0',size);

    /*
    memcpy( generated_stmt , expression, strlen(expression));
    memcpy(&(generated_stmt[strlen(expression)]) ,";",1 );
	*/

    strcat(generated_stmt,expression);
    strcat(generated_stmt,";");


    for(int i =0; i < heapStackCount; i++){
       	symbol_node node = symbol_table[heapStack[i]];
       	strcat(generated_stmt,"delete ");
       	strcat(generated_stmt,(unsigned char *)(node.value_name));
       	strcat(generated_stmt,";");
    }
    heapStackCount = 0;
    return generated_stmt;
}

unsigned char *ifstatement_icg(struct Expression *expression, unsigned char *statement) {



	if(expression->type != NODETYPE_BOOLEAN){
		yyerror("Boolean expression is needed for if-statement.");
	}
    //if ( expression ) statement

    int size = strlen(expression->code) + strlen(statement) +  5;


    unsigned char *generated_stmt = (unsigned char *) malloc(sizeof(unsigned char) * size);
    memset(generated_stmt,'\0',size);

    memcpy(generated_stmt, "if(" , 3 );
    memcpy(&(generated_stmt[3]), expression->code, strlen(expression->code));
    generated_stmt[ 3 + strlen(expression->code)] = ')';
    memcpy(&(generated_stmt[3 + strlen(expression->code) + 1]),statement, strlen(statement));

    return generated_stmt;

}

unsigned char *breakstatement_icg(int sIndex) {
    // break;\0
    int size = 7;
    unsigned char *generated_stmt = (unsigned char *)malloc(sizeof(unsigned char) * size);
    memcpy(generated_stmt,"break;\0",size);
    return generated_stmt;
}

unsigned char *continuestatement_icg(int sIndex) {
    // continue;\0
    int size = 10;
    unsigned char *generated_stmt = (unsigned char *)malloc(sizeof(unsigned char) * size);
    memcpy(generated_stmt,"continue;\0",size);

    return generated_stmt;
}


struct Parameter_List *parameterlist_icg(struct Parameter_List *parameterlist, struct Simple_Expression *simplexpression) {

    struct Parameter_List *parm_list = (struct Parameter_List *)malloc(sizeof(struct Parameter_List));

    // parameterlist , simplexpression
    int size = strlen(parameterlist->code) + strlen(simplexpression->code) + 2;
    unsigned char *generated_stmt = (unsigned char *)malloc(sizeof(unsigned char) * size);
    memset(generated_stmt,'\0',size);
    memcpy(generated_stmt,parameterlist->code,strlen(parameterlist->code));
    generated_stmt[strlen(parameterlist->code)] = ',' ;
    memcpy(&(generated_stmt[strlen(parameterlist->code)+1]),simplexpression->code,strlen(simplexpression->code));

    parm_list->code = generated_stmt;
    parm_list->simple_expression = simplexpression;
    parm_list->type = simplexpression->type;
    parm_list->parameterlist = parameterlist;
    return parm_list;

}

struct Parameter_List *parameterlist_simpleexpression_icg(struct Simple_Expression *simplexpression) {


    struct Parameter_List *parm_list = (struct Parameter_List *)malloc(sizeof(struct Parameter_List));

    parm_list->code = simplexpression->code;
    parm_list->simple_expression = simplexpression;
    parm_list->type = simplexpression->type;
    parm_list->parameterlist = NULL;
    return parm_list;
}

struct Parameter_List *parameterlist_type_icg(int typeIndex) {
    struct Parameter_List *parm_list = (struct Parameter_List *)malloc(sizeof(struct Parameter_List));

    symbol_node typenode =  symbol_table[typeIndex];
    int size = strlen(typenode.value_name) + 2;
    unsigned char *generated_stmt = (unsigned char *)malloc(sizeof(unsigned char) * size);
    memset(generated_stmt,'\0',size);

    memcpy(generated_stmt,typenode.value_name,strlen(typenode.value_name));

    parm_list->type = typenode.node_type;
    parm_list->code = generated_stmt;
    parm_list->parameterlist = NULL;
    return parm_list;
}


struct Function *function_icg(int idSIndex, struct Parameter_List *parameterlist) {



    symbol_node idnode =  symbol_table[idSIndex];
    int namespace = -1;
    if (idnode.namespace_type > -1 && !StackIsEmpty(namespaceStack))
    {
        namespace = namespaceStack->data;
    }



    struct Function *func = (struct Function*)malloc(sizeof(struct Function));

    struct Parameter_List* templist = parameterlist;

    struct function_data tmp_param;
    tmp_param.num_params = 0;
    //count the number of parameters, this is needed to allocate memory to tmp_param.param_types
    while (templist)
    {
        tmp_param.num_params++;
        templist = templist->parameterlist;
    }
    //now set up param_types
    tmp_param.param_types = malloc(sizeof(enum symbol_node_type) * tmp_param.num_params);

    templist = parameterlist; //point to start of list again
    int index = 0;
    while(templist)
    {
        tmp_param.param_types[index] = templist->simple_expression->type;
        templist = templist->parameterlist;
        index++;
    }


    symbol_node compareNode = idnode;
    compareNode.namespace_type = namespace;
    compareNode.param_data = tmp_param;
    int id = find_symbol_node(&compareNode);
    if (id == -1)
    {
        yyerror("No known function definition");
    }

    if (idnode.token_type != TOKENTYPE_PREDEFINED_FUNCTION)
        yyerror(strcat(idnode.value_name, " - is not a predefined function."));

    idnode = symbol_table[id];


    //free(tmp_param.param_types);


    char* generated_function_stmt;
    //No errors, so generate the c++ code:
    if (!StackIsEmpty(namespaceStack) && count_symbols_same_declaration(&idnode))
    {
        symbol_node tmp_node = symbol_table[namespaceStack->data];
        if (idnode.param_data.returns_pointer)
        {
        //    sleep(2);
            int str_length  = strlen(tmp_node.value_name) + 2 + strlen(idnode.value) + strlen(parameterlist->code) + 5 + 1; //2 for ::, 5 for *(), 1 for '/0'
            generated_function_stmt = malloc(sizeof(char) * str_length);
            snprintf(generated_function_stmt, str_length, "*(%s::%s(%s))", tmp_node.value,
                                                                        idnode.value,
                                                                        parameterlist->code);
        }
        else
        {
            int str_length  = strlen(tmp_node.value_name) + 2 + strlen(idnode.value) + strlen(parameterlist->code) + 6;
            generated_function_stmt = malloc(sizeof(char) * str_length);
            snprintf(generated_function_stmt, str_length, "%s::%s(%s)", tmp_node.value,
            idnode.value,
            parameterlist->code);
        }
    }
    else
    {
        if (idnode.param_data.returns_pointer)
        {
            int str_length  = strlen(idnode.value) + strlen(parameterlist->code) + 6;
            generated_function_stmt = malloc(sizeof(char) * str_length);
            snprintf(generated_function_stmt, str_length, "*(%s(%s))", idnode.value, parameterlist->code);
        }
        else
        {
            int str_length  = strlen(idnode.value) + strlen(parameterlist->code) + 6;
            generated_function_stmt = malloc(sizeof(char) * str_length);
            snprintf(generated_function_stmt, str_length, "%s(%s)", idnode.value, parameterlist->code);
        }
    }



    func->type = idnode.node_type;//parameterlist->simple_expression->type;
    func->id_index = idSIndex;
    func->parameterlist = parameterlist;
    func->code = generated_function_stmt;
    return func;


}



struct Function *namespace_function_icg(int namespaceIndex, int operatorIndex,
                                        int idSIndex, struct Parameter_List *parameterlist) {

    symbol_node idnode =  symbol_table[idSIndex];
    symbol_node namespaceNode = symbol_table[namespaceIndex];  //get the namespace name
    symbol_node operatorNode = symbol_table[operatorIndex];  //get the namespace operator


    if (namespaceNode.token_type != TOKENTYPE_NAMESPACE)
    {
        int size = strlen("Unknown namespace \"\"") + strlen(namespaceNode.value_name) + 1; //+1 for null
        char errorMsg[size];
        snprintf(errorMsg, size, "Unknown namespace \"%s\"", namespaceNode.value_name);
        yyerror(errorMsg);
    }


    struct Function *func = (struct Function*)malloc(sizeof(struct Function));

    struct Parameter_List* templist = parameterlist;

    struct function_data tmp_param;
    tmp_param.num_params = 0;
    //count the number of parameters
    while (templist)
    {
        tmp_param.num_params++;
        templist = templist->parameterlist;
    }
    //now set up param_types
    tmp_param.param_types = malloc(sizeof(enum symbol_node_type) * tmp_param.num_params);

    templist = parameterlist;
    int index = 0;
    while(templist)
    {
        tmp_param.param_types[index] = templist->simple_expression->type;
        templist = templist->parameterlist;
        index++;
    }


    symbol_node compareNode = idnode;
    compareNode.namespace_type = namespaceNode.namespace_type;
    compareNode.param_data = tmp_param;
    int id = find_symbol_node(&compareNode);
    if (id == -1)
    {
        yyerror("No known function definition in namespace");
    }

    if (idnode.token_type != TOKENTYPE_PREDEFINED_FUNCTION)
        yyerror(strcat(idnode.value_name, " - is not a predefined function."));

    idnode = symbol_table[id];

    free(tmp_param.param_types);



    char* generated_function_stmt;
    //No errors, so generate the c++ code:
    if (idnode.param_data.returns_pointer)
    {
        int str_length  = strlen(namespaceNode.value_name) + strlen(operatorNode.value_name) +
                          strlen(idnode.value) + strlen(parameterlist->code) + 6;
        generated_function_stmt = malloc(sizeof(char) * str_length);
        snprintf(generated_function_stmt, str_length, "*(%s%s%s(%s))", namespaceNode.value,
                                                                    operatorNode.value_name,
                                                                    idnode.value,
                                                                    parameterlist->code);
    }
    else
    {
        int str_length  = strlen(namespaceNode.value_name) + strlen(operatorNode.value_name) +
                          strlen(idnode.value) + strlen(parameterlist->code) + 6;
        generated_function_stmt = malloc(sizeof(char) * str_length);
        snprintf(generated_function_stmt, str_length, "%s%s%s(%s)", namespaceNode.value,
                                                                    operatorNode.value_name,
                                                                    idnode.value,
                                                                    parameterlist->code);
    }

    func->type = idnode.node_type;//parameterlist->simple_expression->type;
    func->id_index = idSIndex;
    func->parameterlist = parameterlist;
    func->code = generated_function_stmt;
    return func;


}




void print_statement(unsigned char *generated_stmt) {



    //Increase the code size by the size of the code generated here
    long length_stmt = strlen(generated_stmt);
    codeSize += length_stmt ;


    //Create a temoporary string to the size of the code
    unsigned char new_code[codeSize];
    //unsigned char *new_code = (unsigned char *)malloc(sizeof(unsigned char) * codeSize);
    memset(new_code, '\0' , codeSize);

    //Find the $$Declaration$$ marker point in the code
    unsigned char *pointer = strstr(code,"$$ProgramCode$$");


    //Get the character index right before $$Declaration$$ marker
    long markerIndex = (pointer - code);



    /*
     * The template code gets devided into two parts:
     *
     * 1) Part one, is from the beggining of the file till before the "$$Declaration$$" Marker
     * 2) Part two, is from the beggining of the "$$Declaration$$" Marker to end of the file.
     *
     * The new generated statement "declarestmt" should get inserted between these two parts.
     * Thus, We copy the first part to the new buffer. Then we insert the declared statement.
     * Finally, we concatinate the part two.
     *
     * The following code perform this operation
     */
    //Insert the code from begginig till before the $$Declaration$$ marker
    memcpy(new_code,code,markerIndex);

    //Insert the new generated code
    memcpy(&(new_code[markerIndex]),generated_stmt,length_stmt);

    //Insert the rest of code from $$Declaration$$ to the end of file
    memcpy(&(new_code[markerIndex+length_stmt]),&(code[markerIndex]),codeSize-(length_stmt + markerIndex));
    //memcpy(&(new_code[markerIndex+length_stmt]),"pooria\0", 10 );

    code = (unsigned char *) realloc(code,sizeof(unsigned char) * codeSize);


    memcpy(code,new_code,codeSize);

}

void generate_stringliterals_and_fmpointers(){

	 int i;
    for (i=0; i < number_of_nodes; i++) {
        symbol_node node = symbol_table[i];
        if(node.token_type ==TOKENTYPE_STRINGLITERALS
        		|| node.node_type == NODETYPE_FMPOINTER){
			declaration_icg(i);
        }
	}

}


void close_icg(void) {


    char *pointer ;
    generate_stringliterals_and_fmpointers();

    pointer= strstr(code,"$$Declaration$$");
    strncpy (pointer,"               ",15);

    pointer = strstr(code,"$$ProgramCode$$");
    strncpy (pointer,"               ",15);

    pointer = strstr(code,"$$Namespace$$");
    strncpy (pointer,"               ",13);

    fprintf(generatedfile,"%s",code);

    fclose(templatefile);
    fclose(generatedfile);

    free(code);

    StackDelete(&namespaceStack);

}


void using_declaration_icg(int namespaceName)
{



    unsigned char stmt[] = "using namespace ";

    symbol_node namespaceNode = symbol_table[namespaceName];  //get the namespace name

    if (namespaceNode.token_type != TOKENTYPE_NAMESPACE)
    {
        int size = strlen("Unknown namespace \"\"") + strlen(namespaceNode.value_name) + 1; //+1 for null
        char errorMsg[size];
        snprintf(errorMsg, size, "Unknown namespace \"%s\"", namespaceNode.value_name);
        yyerror(errorMsg);
    }


    StackPush(&namespaceStack, namespaceName);

    int size = strlen(stmt) + strlen(namespaceNode.value_name) + 1 + 1 + 2 + 9;


    unsigned char declarestmt[size];	//The size of the C++ code will not be bigger tha 100 character
    memset(declarestmt, '\0' , size * sizeof(unsigned char)); //Set all to EOF; it will be needed for strcat
    strcat(declarestmt,stmt);
    strcat(declarestmt,namespaceNode.value_name);



    strcat(declarestmt,";\n\t");

    //unsigned char declarestmt[] = "int i;\n\t;";
    //At the end we have generated something like: int <node.value_name> ;

    long declaresize = strlen(declarestmt) ;

    //Increase the code size by the size of the code generated here
    codeSize += declaresize ;


    //Create a temoporary string to the size of the code
    unsigned char new_code[codeSize];
    memset(new_code, '\0' , codeSize);

    //Find the $$Namespace$$ marker point in the code
    unsigned char *pointer = strstr(code,"$$Namespace$$");


    //Get the character index right before $$Namespace$$ marker
    long dexindex = (pointer - code);



    //Insert the code from begginig till before the $$Namespace$$ marker
    memcpy(new_code,code,dexindex);

    //Insert the new generated code
    memcpy(&(new_code[dexindex]),declarestmt,declaresize);

    //Insert the rest of code from $$Namespace$$ to the end of file
    memcpy(&(new_code[dexindex+declaresize]),&(code[dexindex]),codeSize-(declaresize + dexindex));



    code = (unsigned char *) realloc(code,sizeof(unsigned char) * codeSize);

    memcpy(code,new_code,codeSize);

    //Print the new generated code
    // Only for debugging purposes
    //printf("\n%s\n",new_code);


}
